///////////////////////////////////////////////////////////////////////////////
// A simple shader that essentially renders a simple depth map, albeit in a
// color channel to make sure it matches the secondLayerShader
///////////////////////////////////////////////////////////////////////////////
shader firstLayerVertex
{
	#version 420 compatibility
	out vec4 position;
	uniform mat4 modelMatrix;
	void main()
	{
		position = gl_ModelViewProjectionMatrix * modelMatrix * gl_Vertex; 
		gl_Position = position; 
	}
}

shader firstLayerFragment
{
	#version 420 compatibility
	in vec4 position; 
	void main()
	{
		gl_FragColor = vec4(position.z / position.w, 0.0, 0.0, 1.0);
	}
}

///////////////////////////////////////////////////////////////////////////////
// For closed objects, we also need to find the "second layer depth map". That
// is, the depths at which a ray from the light would first leave all closed 
// objects and enter open space again. This is obtained with a multi-pass
// algorithm similar to depth-peeling. 
///////////////////////////////////////////////////////////////////////////////
shader secondLayerVertex
{
	#version 420 compatibility
	out vec4 position;
	uniform mat4 modelMatrix;
	void main()
	{
		position = gl_ModelViewProjectionMatrix * modelMatrix * gl_Vertex;
		gl_Position = position;
	}
}

shader secondLayerFragment
{
	#version 400 compatibility
	in vec4 position;
	uniform sampler2D txt;
	uniform int pass;
	void main()
	{
		vec2 t = texelFetch(txt, ivec2(gl_FragCoord.xy), 0).rg;
		float prevSurfaceCount = t.g;
		float prevDepth = t.r;
		float currDepth = position.z / position.w;
		if (pass == 0) {
			if (!gl_FrontFacing) discard;
			gl_FragColor.g = 1.0;
			gl_FragColor.r = currDepth;
		}
		else if (prevSurfaceCount != 0.0){
			if (currDepth > t.r) {
				gl_FragColor.g = prevSurfaceCount + ((gl_FrontFacing) ? 1.0 : -1.0);
				gl_FragColor.r = currDepth;
			}
			else discard;
		}
		else discard;
	}
}

